# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/diagnostics/services/report.proto, blickfeld/diagnostics/services/self_test.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    List,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class ReportGenerateResponse(betterproto.Message):
    """Stream of the encrypted report in chunks of binary data"""

    binary_chunk: bytes = betterproto.bytes_field(1)
    """Binary chunk of the diagnostics report."""


@dataclass(eq=False, repr=False)
class SelfTestRunRequest(betterproto.Message):
    """The self test run can be configured with this request"""

    self_tests: List["_config__.SelfTest"] = betterproto.enum_field(1)
    """
    Specify self tests which should be executed. If not set, the default set
    will be executed.
    """


@dataclass(eq=False, repr=False)
class SelfTestRunResponse(betterproto.Message):
    """
    The response provides the self test report. [NOTE] The method will not
    throw an exception on API level if tests failed or have warnings. The
    success flag in the report must be checked.
    """

    report: "_data__.SelfTestReport" = betterproto.message_field(1)
    """Test report"""


class Report(betterproto.ServiceStub):
    """
    The Report service generates an encrypted diagnostics report containing
    information about the state and health information of the Qb2 device.
    """

    async def async_generate(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["ReportGenerateResponse"]:
        """Returns a stream of the report's binary chunks"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.diagnostics.services.Report/Generate",
            request,
            ReportGenerateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def generate(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["ReportGenerateResponse"]:
        """Returns a stream of the report's binary chunks"""

        loop = asyncio.get_event_loop()
        ait = self.async_generate(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class SelfTest(betterproto.ServiceStub):
    """
    The SelfTest service offers methods to trigger test routines. The routines
    focus on automatic self tests without user interaction. The reports give an
    indication what the potential root cause for an found issue could be.
    """

    async def async_run(
        self,
        *,
        self_tests: Optional[List["_config__.SelfTest"]] = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SelfTestRunResponse":
        """This triggers the default test case routine"""

        self_tests = self_tests or []

        request = SelfTestRunRequest()
        request.self_tests = self_tests

        return await self._unary_unary(
            "/blickfeld.diagnostics.services.SelfTest/Run",
            request,
            SelfTestRunResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def run(
        self,
        *,
        self_tests: Optional[List["_config__.SelfTest"]] = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "SelfTestRunResponse":
        """This triggers the default test case routine"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_run(
                self_tests=self_tests,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/diagnostics/data/self_test_report.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from .. import config as _config__


class SelfTestReportResultFlag(betterproto.Enum):
    """Result flags which indicates if test was sucessful."""

    FLAG_UNSPECIFIED = 0
    """Unknown result."""

    FLAG_SUCCESS = 1
    """The test was successful."""

    FLAG_WARNING = 2
    """
    The test suceeded but it raised some warnings. Please report if this
    happens during production and the root-cause is unknown.
    """

    FLAG_FAILURE = 3
    """The test failed. The tested module is most probably not functional."""


@dataclass(eq=False, repr=False)
class SelfTestReport(betterproto.Message):
    """Report which contains the results of the selected self tests."""

    result_flag: "SelfTestReportResultFlag" = betterproto.enum_field(1)
    """
    Aggregated result flag which is FLAG_SUCCESS if all tests passed without
    warnings
    """

    results: List["SelfTestReportResult"] = betterproto.message_field(2)
    """List of test results"""


@dataclass(eq=False, repr=False)
class SelfTestReportResult(betterproto.Message):
    """Result of test."""

    test: "_config__.SelfTest" = betterproto.enum_field(1)
    """Test case"""

    identifier: str = betterproto.string_field(5)
    """Unique identifier of test case"""

    flag: "SelfTestReportResultFlag" = betterproto.enum_field(2)
    """Flag which states test result"""

    reason: str = betterproto.string_field(3)
    """
    Human-readable reason for test warning or failure. Is not set if test was
    successful.
    """

    duration: float = betterproto.float_field(4)
    """Duration of test run"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/diagnostics/config/self_test.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class SelfTest(betterproto.Enum):
    """
    List of available self tests NOTE: The test list is still incomplete.
    """

    SELF_TEST_UNSPECIFIED = 0
    """No test is specified."""

    SELF_TEST_POINT_CLOUD = 1
    """
    Simple test which starts up the device, requests a point cloud, validates
    its properties and stops the device.
    """

    SELF_TEST_CONNECTION = 2
    """
    Simple connection tests to basic peripherals: PCIe, laser, detector,
    vertical BDC & horizontal BDC. It performs version requests to the modules.
    """

    SELF_TEST_BEAM_DEFLECTION_UNIT = 3
    """Simple check for basic functionality of the beam deflection units"""
